@@ -19,8 +19,8 @@ from .intents import Intent, Intents, RangeSlotList, SlotList, TextSlotList
 from .util import normalize_text, normalize_whitespace
 
 NUMBER_START = re.compile(r"^(\s*-?[0-9]+)")
-
-NON_WORD_START = re.compile(r"^\W+")
+PUNCTUATION = re.compile(r"[.。,，?¿？!！;；:：]+")
+WHITESPACE = re.compile(r"\s+")
 
 
 class HassilError(Exception):
@@ -50,11 +50,8 @@ class MatchEntity:
 
 
 @dataclass
-class MatchContext:
-    """Context passed to match_expression."""
-
-    text: str
-    """Input text remaining to be processed."""
+class MatchSettings:
+    """Settings used in match_expression."""
 
     slot_lists: Dict[str, SlotList] = field(default_factory=dict)
     """Available slot lists mapped by name."""
@@ -62,16 +59,30 @@ class MatchContext:
     expansion_rules: Dict[str, Sentence] = field(default_factory=dict)
     """Available expansion rules mapped by name."""
 
+    ignore_whitespace: bool = False
+    """True if whitespace should be ignored during matching."""
+
+
+@dataclass
+class MatchContext:
+    """Context passed to match_expression."""
+
+    text: str
+    """Input text remaining to be processed."""
+
     entities: List[MatchEntity] = field(default_factory=list)
     """Entities that have been found in input text."""
 
     intent_context: Dict[str, Any] = field(default_factory=dict)
     """Context items from outside or acquired during matching."""
 
+    is_start_of_word: bool = True
+    """True if current text is the start of a word."""
+
     @property
     def is_match(self) -> bool:
-        """True if no text is left that isn't just whitespace or non-word characters"""
-        text = re.sub(r"\W+", "", self.text).strip()
+        """True if no text is left that isn't just whitespace or punctuation"""
+        text = PUNCTUATION.sub("", self.text).strip()
         return not text
 
 
@@ -126,7 +137,7 @@ def recognize_all(
     default_response: Optional[str] = "default",
 ) -> Iterable[RecognizeResult]:
     """Return all matches for input text/words against a collection of intents."""
-    text = normalize_text(text)
+    text = normalize_text(text).strip()
 
     if skip_words is None:
         skip_words = intents.skip_words
@@ -137,7 +148,11 @@ def recognize_all(
     if skip_words:
         text = _remove_skip_words(text, skip_words)
 
-    text += " "
+    if intents.settings.ignore_whitespace:
+        text = WHITESPACE.sub("", text)
+    else:
+        # Artifical word boundary
+        text += " "
 
     if slot_lists is None:
         slot_lists = intents.slot_lists
@@ -157,6 +172,12 @@ def recognize_all(
     if intent_context is None:
         intent_context = {}
 
+    settings = MatchSettings(
+        slot_lists=slot_lists,
+        expansion_rules=expansion_rules,
+        ignore_whitespace=intents.settings.ignore_whitespace,
+    )
+
     # Check sentence against each intent.
     # This should eventually be done in parallel.
     for intent in intents.intents.values():
@@ -165,100 +186,98 @@ def recognize_all(
                 # Create initial context
                 match_context = MatchContext(
                     text=text,
-                    slot_lists=slot_lists,
-                    expansion_rules=expansion_rules,
                     intent_context=intent_context,
                 )
-                maybe_match_contexts = match_expression(match_context, intent_sentence)
+                maybe_match_contexts = match_expression(
+                    settings, match_context, intent_sentence
+                )
                 for maybe_match_context in maybe_match_contexts:
-                    if maybe_match_context.is_match:
-                        skip_match = False
+                    if not maybe_match_context.is_match:
+                        continue
 
-                        # Verify excluded context
-                        if intent_data.excludes_context:
-                            for (
-                                context_key,
-                                context_value,
-                            ) in intent_data.excludes_context.items():
-                                actual_value = maybe_match_context.intent_context.get(
-                                    context_key
-                                )
-                                if actual_value == context_value:
-                                    # Exact match to context value
-                                    skip_match = True
-                                    break
+                    skip_match = False
 
-                                if (
-                                    isinstance(
-                                        context_value, collections.abc.Collection
-                                    )
-                                    and not isinstance(context_value, str)
-                                    and (actual_value in context_value)
-                                ):
-                                    # Actual value was in context value list
-                                    skip_match = True
-                                    break
-
-                        # Verify required context
-                        if (not skip_match) and intent_data.requires_context:
-                            for (
-                                context_key,
-                                context_value,
-                            ) in intent_data.requires_context.items():
-                                actual_value = maybe_match_context.intent_context.get(
-                                    context_key
-                                )
-
-                                if (
-                                    actual_value == context_value
-                                    and context_value is not None
-                                ):
-                                    # Exact match to context value, except when context value is required and not provided
-                                    continue
-
-                                if context_value is None and actual_value is not None:
-                                    # Any value matches, as long as it's set
-                                    continue
-
-                                if (
-                                    isinstance(
-                                        context_value, collections.abc.Collection
-                                    )
-                                    and not isinstance(context_value, str)
-                                    and (actual_value in context_value)
-                                ):
-                                    # Actual value was in context value list
-                                    continue
-
-                                # Did not match required context
+                    # Verify excluded context
+                    if intent_data.excludes_context:
+                        for (
+                            context_key,
+                            context_value,
+                        ) in intent_data.excludes_context.items():
+                            actual_value = maybe_match_context.intent_context.get(
+                                context_key
+                            )
+                            if actual_value == context_value:
+                                # Exact match to context value
                                 skip_match = True
                                 break
 
-                        if skip_match:
-                            # Intent context did not match
-                            continue
+                            if (
+                                isinstance(context_value, collections.abc.Collection)
+                                and not isinstance(context_value, str)
+                                and (actual_value in context_value)
+                            ):
+                                # Actual value was in context value list
+                                skip_match = True
+                                break
 
-                        # Add fixed entities
-                        for slot_name, slot_value in intent_data.slots.items():
-                            maybe_match_context.entities.append(
-                                MatchEntity(name=slot_name, value=slot_value, text="")
+                    # Verify required context
+                    if (not skip_match) and intent_data.requires_context:
+                        for (
+                            context_key,
+                            context_value,
+                        ) in intent_data.requires_context.items():
+                            actual_value = maybe_match_context.intent_context.get(
+                                context_key
                             )
 
-                        # Return the first match
-                        response = default_response
-                        if intent_data.response is not None:
-                            response = intent_data.response
+                            if (
+                                actual_value == context_value
+                                and context_value is not None
+                            ):
+                                # Exact match to context value, except when context value is required and not provided
+                                continue
 
-                        yield RecognizeResult(
-                            intent=intent,
-                            entities={
-                                entity.name: entity
-                                for entity in maybe_match_context.entities
-                            },
-                            entities_list=maybe_match_context.entities,
-                            response=response,
+                            if context_value is None and actual_value is not None:
+                                # Any value matches, as long as it's set
+                                continue
+
+                            if (
+                                isinstance(context_value, collections.abc.Collection)
+                                and not isinstance(context_value, str)
+                                and (actual_value in context_value)
+                            ):
+                                # Actual value was in context value list
+                                continue
+
+                            # Did not match required context
+                            skip_match = True
+                            break
+
+                    if skip_match:
+                        # Intent context did not match
+                        continue
+
+                    # Add fixed entities
+                    for slot_name, slot_value in intent_data.slots.items():
+                        maybe_match_context.entities.append(
+                            MatchEntity(name=slot_name, value=slot_value, text="")
                         )
 
+                    # Return each match
+                    response = default_response
+                    if intent_data.response is not None:
+                        response = intent_data.response
+
+                    yield RecognizeResult(
+                        intent=intent,
+                        entities={
+                            entity.name: entity
+                            for entity in maybe_match_context.entities
+                        },
+                        entities_list=maybe_match_context.entities,
+                        response=response,
+                    )
+
 
 def is_match(
     text: str,
@@ -268,14 +287,19 @@ def is_match(
     skip_words: Optional[Iterable[str]] = None,
     entities: Optional[Dict[str, Any]] = None,
     intent_context: Optional[Dict[str, Any]] = None,
+    ignore_whitespace: bool = False,
 ) -> Optional[MatchContext]:
     """Return the first match of input text/words against a sentence expression."""
-    text = normalize_text(text)
+    text = normalize_text(text).strip()
 
     if skip_words:
         text = _remove_skip_words(text, skip_words)
 
-    text = text + " "
+    if ignore_whitespace:
+        text = WHITESPACE.sub("", text)
+    else:
+        # Artifical word boundary
+        text += " "
 
     if slot_lists is None:
         slot_lists = {}
@@ -286,14 +310,18 @@ def is_match(
     if intent_context is None:
         intent_context = {}
 
-    match_context = MatchContext(
-        text=text,
+    settings = MatchSettings(
         slot_lists=slot_lists,
         expansion_rules=expansion_rules,
+        ignore_whitespace=ignore_whitespace,
+    )
+
+    match_context = MatchContext(
+        text=text,
         intent_context=intent_context,
     )
 
-    for maybe_match_context in match_expression(match_context, sentence):
+    for maybe_match_context in match_expression(settings, match_context, sentence):
         if maybe_match_context.is_match:
             return maybe_match_context
 
@@ -316,63 +344,81 @@ def _remove_skip_words(text: str, skip_words: Iterable[str]) -> str:
 
 
 def match_expression(
-    context: MatchContext, expression: Expression
+    settings: MatchSettings, context: MatchContext, expression: Expression
 ) -> Iterable[MatchContext]:
-    """Yield matching contexts for an expresion"""
+    """Yield matching contexts for an expression"""
     if isinstance(expression, TextChunk):
         chunk: TextChunk = expression
-        chunk_text = chunk.text.lstrip()
+
+        if settings.ignore_whitespace:
+            # Remove all whitespace
+            chunk_text = WHITESPACE.sub("", chunk.text)
+            context_text = WHITESPACE.sub("", context.text)
+        else:
+            # Keep whitespace
+            chunk_text = chunk.text
+            context_text = context.text
+
+            if context.is_start_of_word:
+                # Ignore extra whitespace at the beginning of chunk and text
+                # since we know we're at the start of a word.
+                chunk_text = chunk_text.lstrip()
+                context_text = context_text.lstrip()
 
         if chunk.is_empty:
             # Skip empty chunk
             yield context
-        elif context.text.startswith(chunk_text):
+        elif context_text.startswith(chunk_text):
             # Successful match for chunk
-            context_text = context.text[len(chunk_text) :]
-            context_text = context_text.lstrip()
+            context_text = context_text[len(chunk_text) :]
             yield MatchContext(
                 text=context_text,
+                # must use chunk.text because it hasn't been stripped
+                is_start_of_word=chunk.text.endswith(" "),
                 # Copy over
-                slot_lists=context.slot_lists,
-                expansion_rules=context.expansion_rules,
+                entities=context.entities,
+                intent_context=context.intent_context,
+            )
+        elif chunk_text.isspace():
+            yield MatchContext(
+                is_start_of_word=True,
+                # Copy over
+                text=context_text,
                 entities=context.entities,
                 intent_context=context.intent_context,
             )
         else:
-            # Remove non-word characters and try again
-            match = NON_WORD_START.match(context.text)
-            if match is not None:
-                context_text = context.text[len(match[0]) :].lstrip()
-                if context_text.startswith(chunk_text):
-                    context_text = context_text[len(chunk_text) :]
-                    context_text = context_text.lstrip()
-                    yield MatchContext(
-                        text=context_text,
-                        # Copy over
-                        slot_lists=context.slot_lists,
-                        expansion_rules=context.expansion_rules,
-                        entities=context.entities,
-                        intent_context=context.intent_context,
-                    )
-
+            # Remove punctuation and try again
+            context_text = PUNCTUATION.sub(" ", context.text).lstrip()
+            if context_text.startswith(chunk_text):
+                context_text = context_text[len(chunk_text) :]
+                yield MatchContext(
+                    text=context_text,
+                    # Copy over
+                    entities=context.entities,
+                    intent_context=context.intent_context,
+                    is_start_of_word=context.is_start_of_word,
+                )
     elif isinstance(expression, Sequence):
         seq: Sequence = expression
         if seq.type == SequenceType.ALTERNATIVE:
             # Any may match (words | in | alternative)
             # NOTE: [optional] = (optional | )
             for item in seq.items:
-                yield from match_expression(context, item)
+                yield from match_expression(settings, context, item)
 
         elif seq.type == SequenceType.GROUP:
-            # All must match (words in group)
             if seq.items:
+                # All must match (words in group)
                 group_contexts = [context]
                 for item in seq.items:
                     # Next step
                     group_contexts = [
                         item_context
                         for group_context in group_contexts
-                        for item_context in match_expression(group_context, item)
+                        for item_context in match_expression(
+                            settings, group_context, item
+                        )
                     ]
                     if not group_contexts:
                         break
@@ -385,24 +431,23 @@ def match_expression(
     elif isinstance(expression, ListReference):
         # {list}
         list_ref: ListReference = expression
-        if (not context.slot_lists) or (list_ref.list_name not in context.slot_lists):
+        if (not settings.slot_lists) or (list_ref.list_name not in settings.slot_lists):
             raise MissingListError(f"Missing slot list {{{list_ref.list_name}}}")
 
-        slot_list = context.slot_lists[list_ref.list_name]
+        slot_list = settings.slot_lists[list_ref.list_name]
         if isinstance(slot_list, TextSlotList):
-            text_list: TextSlotList = slot_list
-
             if context.text:
+                text_list: TextSlotList = slot_list
                 # Any value may match
                 for slot_value in text_list.values:
                     value_contexts = match_expression(
+                        settings,
                         MatchContext(
                             # Copy over
                             text=context.text,
-                            slot_lists=context.slot_lists,
-                            expansion_rules=context.expansion_rules,
                             entities=context.entities,
                             intent_context=context.intent_context,
+                            is_start_of_word=context.is_start_of_word,
                         ),
                         slot_value.text_in,
                     )
@@ -428,24 +473,22 @@ def match_expression(
                                 },
                                 # Copy over
                                 text=value_context.text,
-                                slot_lists=value_context.slot_lists,
-                                expansion_rules=value_context.expansion_rules,
+                                is_start_of_word=context.is_start_of_word,
                             )
                         else:
                             yield MatchContext(
                                 entities=entities,
                                 # Copy over
                                 text=value_context.text,
-                                slot_lists=value_context.slot_lists,
-                                expansion_rules=value_context.expansion_rules,
                                 intent_context=value_context.intent_context,
+                                is_start_of_word=context.is_start_of_word,
                             )
 
         elif isinstance(slot_list, RangeSlotList):
-            # List that represents a number range.
-            # Numbers must currently be digits ("1" not "one").
-            range_list: RangeSlotList = slot_list
             if context.text:
+                # List that represents a number range.
+                # Numbers must currently be digits ("1" not "one").
+                range_list: RangeSlotList = slot_list
                 number_match = NUMBER_START.match(context.text)
                 if number_match is not None:
                     number_text = number_match[1]
@@ -469,12 +512,11 @@ def match_expression(
                         ]
 
                         yield MatchContext(
-                            text=context.text[len(number_text) :].lstrip(),
+                            text=context.text[len(number_text) :],
                             entities=entities,
                             # Copy over
-                            slot_lists=context.slot_lists,
-                            expansion_rules=context.expansion_rules,
                             intent_context=context.intent_context,
+                            is_start_of_word=context.is_start_of_word,
                         )
 
         else:
@@ -483,13 +525,17 @@ def match_expression(
     elif isinstance(expression, RuleReference):
         # <rule>
         rule_ref: RuleReference = expression
-        if (not context.expansion_rules) or (
-            rule_ref.rule_name not in context.expansion_rules
+        if (not settings.expansion_rules) or (
+            rule_ref.rule_name not in settings.expansion_rules
         ):
             raise MissingRuleError(f"Missing expansion rule <{rule_ref.rule_name}>")
 
         yield from match_expression(
-            context, context.expansion_rules[rule_ref.rule_name]
+            settings, context, settings.expansion_rules[rule_ref.rule_name]
         )
     else:
         raise ValueError(f"Unexpected expression: {expression}")
+
+
+def _normalize_whitespace(text: str) -> str:
+    return WHITESPACE.sub(" ", text)
