@@ -20,9 +20,9 @@ def test_whitespace():
     assert is_match("  turn      on the     lights", sentence)
 
 
-def test_skip_nonword():
+def test_skip_punctuation():
     sentence = parse_sentence("turn on the lights")
-    assert is_match("turn ! on @ the # lights $", sentence)
+    assert is_match("turn ! on ? the, lights.", sentence)
 
 
 def test_skip_words():
@@ -47,7 +47,7 @@ def test_optional_plural():
 
 
 def test_group_plural():
-    sentence = parse_sentence("give me the penn(y | ies)")
+    sentence = parse_sentence("give me the penn(y|ies)")
     assert is_match("give me the penny", sentence)
     assert is_match("give me the pennies", sentence)
 
@@ -82,17 +82,22 @@ def test_rule_prefix_suffix():
     assert is_match(
         "turn off abc-kitchen-123",
         sentence,
-        expansion_rules={"area": parse_sentence("[the] kitchen")},
+        expansion_rules={"area": parse_sentence("[the ]kitchen")},
     )
 
 
 def test_alternative_whitespace():
-    sentence = parse_sentence("(start | stopp)ed")
+    sentence = parse_sentence("(start|stopp)ed")
     assert is_match("started", sentence)
     assert is_match("stopped", sentence)
 
 
-def test_no_allow_templatekj():
+def test_alternative_whitespace_2():
+    sentence = parse_sentence("set brightness to ( minimum | lowest)")
+    assert is_match("set brightness to lowest", sentence)
+
+
+def test_no_allow_template():
     sentence = parse_sentence("turn off {name}")
     names = TextSlotList.from_strings(["light[s]"])
     assert is_match("turn off lights", sentence, slot_lists={"name": names})
@@ -100,3 +105,8 @@ def test_no_allow_templatekj():
     names = TextSlotList.from_strings(["light[s]"], allow_template=False)
     assert not is_match("turn off lights", sentence, slot_lists={"name": names})
     assert is_match("turn off light[s]", sentence, slot_lists={"name": names})
+
+
+def test_no_whitespace_fails():
+    sentence = parse_sentence("this is a test")
+    assert not is_match("thisisatest", sentence)
